/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 OW2 Consortium,
 * http://www.ow2.org/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.plugin.jbiplugin;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.artifact.versioning.InvalidVersionSpecificationException;
import org.apache.maven.artifact.versioning.VersionRange;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.codehaus.plexus.util.interpolation.MapBasedValueSource;
import org.codehaus.plexus.util.interpolation.ObjectBasedValueSource;
import org.codehaus.plexus.util.interpolation.RegexBasedInterpolator;
import org.ow2.petals.jbi.descriptor.original.generated.Component.SharedLibrary;
import org.ow2.petals.plugin.jbiplugin.util.PropertiesInterpolationValueSource;

/**
 * This class is used to give the main configuration parameters of all JBI goals
 * needing artifact configuration.
 * 
 * @author cdeneux - Capgemini Sud
 */
public abstract class JBIAbstractConfigurableMojo extends JBIAbstractMojo {

    protected final static String JBIDESCRIPTOR_ZIPENTRY_NAME = "META-INF/jbi.xml";

    /**
     * Flag to activate JBI archive configuration when packaging it. If JBI
     * identifiers mapping configuration file is not found, no configuration is
     * applied.
     * 
     * @parameter expression="true"
     * @editable true
     * @required
     * @description Flag to activate JBI archive configuration when packaging
     *              it. If JBI identifiers mapping configuration file is not
     *              found, no configuration is applied.
     */
    protected boolean includeConfiguration;

    /**
     * Mapping pattern of the all shared library identifiers used inside a JBI
     * component.
     * 
     * @parameter expression="$${artifactId}"
     * @editable true
     * @required
     * @description If the JBI identifier of a shared library inside a JBI
     *              component must be computed, sets the mapping pattern of the
     *              all shared library identifiers to used inside the JBI
     *              descriptor of a component. Example of usage (caution to the
     *              usage of "$" and "$$"): <sharedLibraryNameMappingInComponent
     *              >$${artifactId}</sharedLibraryNameMappingInComponent>. Some
     *              extra pattern are defined: "version.major" is the major part
     *              of the version.
     */
    protected String sharedLibraryNameMappingInComponent;

    /**
     * JBI identifiers mapping configuration file.
     * 
     * @parameter expression="${jbiIdentifiersMappingFile}"
     *            default-value="jbi-identifiers-mapping-file.properties"
     * @editable true
     * @description <p>
     *              JBI identifiers mapping configuration file. Must be
     *              available through the plugin classpath.
     *              </p>
     *              <p>
     *              Each file line contains a key and a value. The value is used
     *              as expression defining the JBI identifier of the artifact
     *              matching the key.
     *              </p>
     *              <p>
     *              The key is composed of 4 mandatory fields separated by ":"
     *              (caution to use the escape character), each field can
     *              contain the value "*" to match any value:
     *              <ol>
     *              <li>groupId,</li>
     *              <li>artifactId,</li>
     *              <li>version,</li>
     *              <li>packaging: one of jbi-component, jbi-shared-library,
     *              jbi-service-unit, jbi-service-assembly.</li>
     *              </ol>
     *              </p>
     *              <p>
     *              Example, the following mapping rules define that all
     *              component identifiers contain the major version part except
     *              for the BC Soap:
     *              <ul>
     *              <li>
     *              <code>org.ow2.petals\:petals-bc-soap\:*\:jbi-component = ${artifactId}</code>
     *              ,</li>
     *              <li>
     *              <code>*\:*\:*\:jbi-component = ${artifactId}-${version.major}</code>
     *              </li>
     *              </ul>
     *              </p>
     */
    protected String jbiIdentifiersMappingFile;

    /**
     * JBI identifiers mapping configuration file. If it does not exist through
     * the classpath, the file is look for according to this URL.
     * 
     * @parameter expression="${jbiIdentifiersMappingFileURL}"
     * @editable true
     * @description <p>
     *              JBI identifiers mapping configuration file. If it does not
     *              exist through the classpath, the file is look for according
     *              to this URL.
     *              </p>
     * @see {@link #jbiIdentifiersMappingFile}.
     */
    protected URL jbiIdentifiersMappingFileURL;

    /**
     * Content of the JBI identifiers mapping configuration file as properties.
     */
    final private Properties jbiIdentifiersMappingExpressions = new Properties();

    /**
     * JBI shared library list mapping configuration file.
     * 
     * @parameter expression="${jbiSharedLibrariesMappingFile}" default-value
     *            ="jbi-shared-libraries-list-mapping-file.properties"
     * @editable true
     * @description <p>
     *              JBI shared library list mapping configuration file. Must be
     *              available through the plugin classpath.
     *              </p>
     *              <p>
     *              Each file line contains a key and a value. The value is used
     *              as a Maven artifact to use as shared library by the artifact
     *              matching the key. Use one line per shared library.
     *              </p>
     *              <p>
     *              The key is composed of 4 mandatory fields separated by ":"
     *              (caution to use the escape character), each field (except
     *              the fourth) can contain the value "*" to match any value:
     *              <ol>
     *              <li>groupId,</li>
     *              <li>artifactId,</li>
     *              <li>version,</li>
     *              <li>ordering number: in case of several shared library this
     *              field will define the declaration order of shared libraries.
     *              In case of only one sahred library use the value "1".</li>
     *              </ol>
     *              </p>
     *              <p>
     *              The value is composed of 3 mandatory fields separated by ":"
     *              defining a Maven artifact to use as shared library:
     *              <ol>
     *              <li>groupId,</li>
     *              <li>artifactId,</li>
     *              <li>version.</li>
     *              </ol>
     *              </p>
     *              <p>
     *              Example, the following mapping rules define that all
     *              versions of the BC JMS will use the shared library including
     *              the ActiveMQ JMS client and an other one:
     *              <ul>
     *              <li>
     *              <code>org.ow2.petals\:petals-bc-jms\:*\:1 = org.ow2.petals:petals-sl-jms-activemq:5.2.0</code>
     *              </li>
     *              <li>
     *              <code>org.ow2.petals\:petals-bc-jms\:*\:2 = org.ow2.petals:petals-sl-jms-otherjmsprovider:1.0.0</code>
     *              </li>
     *              </ul>
     *              </p>
     */
    protected String jbiSharedLibrariesMappingFile;

    /**
     * JBI shared library list mapping configuration file. If it does not exist
     * through the classpath, the file is look for according to this URL.
     * 
     * @parameter expression="${jbiSharedLibrariesMappingFileURL}"
     * @editable true
     * @description <p>
     *              JBI shared library list mapping configuration file. If it
     *              does not exist through the classpath, the file is look for
     *              according to this URL.
     *              </p>
     * @see {@link #jbiSharedLibrariesMappingFile}.
     */
    protected URL jbiSharedLibrariesMappingFileURL;

    /**
     * Content of the JBI shared libraries mapping configuration file as
     * properties.
     */
    final private Properties jbiSharedLibrariesMappingExpressions = new Properties();

    /**
     * JBI versions mapping configuration file.
     * 
     * @parameter expression="${jbiVersionsMappingFile}"
     *            default-value="jbi-versions-mapping-file.properties"
     * @editable true
     * @description <p>
     *              JBI versions mapping configuration file. Must be available
     *              through the plugin classpath.
     *              </p>
     *              <p>
     *              Each file line contains a key and a value. The value is used
     *              as expression defining the JBI version of the artifact
     *              matching the key.
     *              </p>
     *              <p>
     *              The key is composed of 4 mandatory fields separated by ":"
     *              (caution to use the escape character), each field can
     *              contain the value "*" to match any value:
     *              <ol>
     *              <li>groupId,</li>
     *              <li>artifactId,</li>
     *              <li>version,</li>
     *              <li>packaging: one of jbi-component, jbi-shared-library,
     *              jbi-service-unit, jbi-service-assembly.</li>
     *              </ol>
     *              </p>
     *              <p>
     *              Example, the following mapping rules define that all shared
     *              libraries version identifiers are the major version:
     *              <ul>
     *              <li>
     *              <code>*\:*\:*\:jbi-shared-library = ${version.major}</code></li>
     *              </ul>
     *              </p>
     */
    protected String jbiVersionsMappingFile;

    /**
     * JBI versions mapping configuration file. If it does not exist through the
     * classpath, the file is look for according to this URL.
     * 
     * @parameter expression="${jbiVersionsMappingFileURL}"
     * @editable true
     * @description <p>
     *              JBI versions mapping configuration file. If it does not
     *              exist through the classpath, the file is look for according
     *              to this URL.
     *              </p>
     * @see {@link #jbiIdentifiersMappingFile}.
     */
    protected URL jbiVersionsMappingFileURL;

    /**
     * Content of the JBI versions mapping configuration file as properties.
     */
    final private Properties jbiVersionsMappingExpressions = new Properties();

    /***
     * <p>
     * Compute the JBI identifier of the artifact according to:
     * <ul>
     * <li>
     * the artifact match a pattern according to:
     * <ul>
     * <li>the configuration mode (<code>includeConfiguration</code>)</li>
     * <li>the artifact matches a pattern in the JBI identifiers mapping
     * configuration file (<code>jbiIdentifiersMappingFile</code>)</li>
     * </ul>
     * </li>
     * <li>a default expression (<code>expression</code>)</li>
     * </ul>
     * </p>
     * 
     * @param expression
     *            The default expression defining the JBI identifier of the
     *            artifact
     * @param artifact
     *            Artifact for which a JBI identifier must be defined
     * @return The computed JBI identifier
     */
    protected String evaluateJBIIdentifier(final String expression, final Artifact artifact)
            throws MojoExecutionException {

        String artifactJBIIdentifier = null;
        if (this.includeConfiguration) {
            artifactJBIIdentifier = this.getJbiDescriptorValue(artifact,
                    this.jbiIdentifiersMappingExpressions);
        }
        if (artifactJBIIdentifier == null) {
            artifactJBIIdentifier = this.evaluateFileNameMapping(expression, artifact);
        }

        return artifactJBIIdentifier;
    }

    /***
     * <p>
     * Compute the JBI version of the artifact according to:
     * <ul>
     * <li>
     * the artifact match a pattern according to:
     * <ul>
     * <li>the configuration mode (<code>includeConfiguration</code>)</li>
     * <li>the artifact matches a pattern in the JBI versions mapping
     * configuration file (<code>jbiVersionsMappingFile</code>)</li>
     * </ul>
     * </li>
     * <li>a default expression (<code>expression</code>)</li>
     * </ul>
     * </p>
     * 
     * @param expression
     *            The default expression defining the JBI version of the
     *            artifact
     * @param artifact
     *            Artifact for which a JBI version must be defined
     * @return The computed JBI version
     */
    protected String evaluateJBIVersion(final String expression, final Artifact artifact)
            throws MojoExecutionException {

        String artifactJBIIdentifier = null;
        if (this.includeConfiguration) {
            artifactJBIIdentifier = this.getJbiDescriptorValue(artifact,
                    this.jbiVersionsMappingExpressions);
        }
        if (artifactJBIIdentifier == null) {
            artifactJBIIdentifier = this.evaluateFileNameMapping(expression, artifact);
        }

        return artifactJBIIdentifier;
    }

    /***
     * <p>
     * Compute the JBI shared libraries list of a component according to:
     * <ul>
     * <li>
     * the artifact match a pattern according to:
     * <ul>
     * <li>the configuration mode (<code>includeConfiguration</code>)</li>
     * <li>the artifact matches a pattern in the JBI shared libraries list
     * mapping configuration file (<code>jbiSharedLibrariesMappingFile</code>)</li>
     * </ul>
     * </li>
     * <li>the default shared library list provided by the original POM file, if
     * existing, on which the mapping defined by the parameter
     * <code>sharedLibraryNameMappingInComponent</code> is applied.</li>
     * </ul>
     * </p>
     * 
     * @param artifact
     *            Artifact for which a JBI shared libraries list must be defined
     * @return The computed JBI shared libraries list
     */
    protected List<SharedLibrary> evaluateSharedLibraries(final Artifact artifact)
            throws MojoExecutionException {

        List<SharedLibrary> sharedLibraries = null;
        if (this.includeConfiguration) {
            sharedLibraries = this.getSharedLibraries(artifact);
        }

        if (sharedLibraries == null) {
            sharedLibraries = this.getSharedLibrariesFromProject();
        }

        return sharedLibraries;
    }

    /**
     * Returns the MavenProject associated to the artifact to configure. In case
     * of the goal "jbi-package", it's the project itself. In case of the goal
     * "jbi-configure", it's the project associated to the provided artifact.
     * 
     * @return
     */
    protected abstract MavenProject getMavenProject();

    /**
     * Evaluate an expression replacing artifact properties by their values.
     * 
     * @param expression
     *            Expression containing artifact property names
     * @param artifact
     *            Artifact containing property values
     * @return
     */
    private String evaluateFileNameMapping(final String expression, final Artifact artifact) {

        Properties props = new Properties();

        // Extact major part of the artifact version
        final String artifactVersion = artifact.getVersion();

        if (artifactVersion.contains(".")) {
            props.put("version.major", artifactVersion.substring(0, artifactVersion.indexOf('.')));
        } else if (artifactVersion.contains("-")) {
            props.put("version.major", artifactVersion.substring(artifactVersion.indexOf('-')));
        } else {
            props.put("version.major", artifactVersion);
        }

        return evaluateFileNameMapping(expression, artifact, props);
    }

    /**
     * Evaluates Filename Mapping (source code copied from
     * maven-assembly-plugin)
     * 
     * @param expression
     * @param artifact
     * @return mapped expression
     * @throws AssemblyFormattingException
     * @throws org.apache.maven.plugin.MojoExecutionException
     */
    // TODO (CDE): Rework to simplify after fixing of MNG-3558
    private String evaluateFileNameMapping(final String expression, final Artifact artifact,
            final Properties properties) {

        this.debug("Evaluating expression: " + expression);

        String value = expression.replaceAll("\\$\\{dollarSign\\}", "\\$");

        // FIXME: This is BAD! Accessors SHOULD NOT change the behavior of the
        // object.
        artifact.isSnapshot();

        RegexBasedInterpolator interpolator = new RegexBasedInterpolator();

        if (artifact != null) {
            interpolator.addValueSource(new ObjectBasedValueSource(artifact));
            interpolator.addValueSource(new ObjectBasedValueSource(artifact.getArtifactHandler()));
        }
        if (properties != null) {
            interpolator.addValueSource(new MapBasedValueSource(properties));
        }

        Properties classifierMask = new Properties();
        classifierMask.setProperty("classifier", "");

        interpolator.addValueSource(new PropertiesInterpolationValueSource(classifierMask));

        value = interpolator.interpolate(value, "__artifact");

        this.debug("Expression evaluated to: " + value);

        return value;
    }

    /**
     * Compute a JBI descriptor value according to a mapping pattern associated
     * to the MAven artifact in mapping configuration files.
     * 
     * @return JBI descriptor value of the artifact, <code>null</code> if not
     *         match found.
     */
    private String getJbiDescriptorValue(final Artifact artifact,
            final Properties jbiMappingExpressions) throws MojoExecutionException {

        String res = null;

        final String mappingExpr = JBIAbstractConfigurableMojo.getMappingExpression(artifact,
                jbiMappingExpressions);
        if (mappingExpr != null) {
            res = this.evaluateFileNameMapping(mappingExpr, artifact);
        }

        return res;
    }

    /**
     * <p>
     * Compute the JBI shared libraries of the artifact according the JBI shared
     * libraries mapping configuration file.
     * </p>
     * <p>
     * Note: The JBI shared library identifiers are also computed according to
     * the following priority order:
     * <ol>
     * <li>if a JBI shared library identifier mapping exists, it is used,</li>
     * <li>otherwise the JBI identifier is defined according to the parameter
     * <code>sharedLibraryNameMappingInComponent</code>.</li>
     * </ol>
     * </p>
     * 
     * @return List of shared libraries (as JBI descriptor element) of the
     *         artifact, <code>null</code> if not match found.
     */
    private List<SharedLibrary> getSharedLibraries(final Artifact artifact)
            throws MojoExecutionException {

        try {
            final List<String> sharedLibrariesStr = this
                    .getSharedLibrariesMappingExpression(artifact);

            if (sharedLibrariesStr != null) {
                final List<SharedLibrary> sharedLibraries = new ArrayList<SharedLibrary>();

                for (final String sharedLibraryStr : sharedLibrariesStr) {

                    final StringTokenizer st = new StringTokenizer(sharedLibraryStr, ":");
                    if (st.countTokens() < 3) {
                        throw new MojoExecutionException(
                                "Missing field in the shared library definition, 3 fields are mandatory: "
                                        + sharedLibraryStr);
                    }
                    final String groupId = st.nextToken();
                    final String artifactId = st.nextToken();
                    final String version = st.nextToken();

                    final Artifact sharedLibraryArtifact = this.artifactFactory
                            .createDependencyArtifact(groupId, artifactId, VersionRange
                                    .createFromVersionSpec(version), JBIAbstractMojo.PACKAGING_SL,
                                    null, null);

                    sharedLibraries.add(this.artifactToSharedLibrary(sharedLibraryArtifact));
                }

                return sharedLibraries;
            } else {
                return null;
            }
        } catch (final InvalidVersionSpecificationException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    /**
     * <p>
     * Compute the JBI shared libraries of the artifact according the POM file.
     * </p>
     * <p>
     * Note: The JBI shared library identifiers are also computed according to
     * the following priority order:
     * <ol>
     * <li>if a JBI shared library identifier mapping exists, it is used,</li>
     * <li>otherwise the JBI identifier is defined according to the parameter
     * <code>sharedLibraryNameMappingInComponent</code>.</li>
     * </ol>
     * </p>
     * 
     * @return List of shared libraries (as JBI descriptor element) of the
     *         artifact.
     */
    private List<SharedLibrary> getSharedLibrariesFromProject() throws MojoExecutionException {

        try {
            final MavenProject artifactProject = this.getMavenProject();
            final List<Artifact> slDependencies = this.getSharedLibraries(artifactProject);
            final List<SharedLibrary> sharedLibraries = new ArrayList<SharedLibrary>();
            for (final Artifact slDependency : slDependencies) {

                final Artifact sharedLibraryArtifact = this.artifactFactory
                        .createDependencyArtifact(slDependency.getGroupId(), slDependency
                                .getArtifactId(), VersionRange.createFromVersionSpec(slDependency
                                .getVersion()), JBIAbstractMojo.PACKAGING_SL, null, null);

                sharedLibraries.add(this.artifactToSharedLibrary(sharedLibraryArtifact));
            }

            return sharedLibraries;
        } catch (final InvalidVersionSpecificationException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    /**
     * Convert a shared library as a Maven artifact to the associated element of
     * the component JBI descriptor. The shared library identifier is computed
     * according to the defined configuration and mapping rules.
     * 
     * @param artifact
     * @return
     */
    protected SharedLibrary artifactToSharedLibrary(final Artifact artifact)
            throws MojoExecutionException {
        final SharedLibrary sharedLibrary = new SharedLibrary();
        final String slContent = this.evaluateJBIIdentifier(
                this.sharedLibraryNameMappingInComponent, artifact);
        sharedLibrary.setContent(slContent);
        sharedLibrary.setVersion(this.evaluateJBIVersion(artifact.getVersion(), artifact));

        return sharedLibrary;
    }

    /**
     * Find a mapping expression associated to the provided Maven artifact in a
     * mapping expression set (ie: mapping configuration file).
     * 
     * @return The mapping expression, <code>null</code> if not match found.
     */
    private static String getMappingExpression(final Artifact artifact,
            final Properties jbiMappingExpressions) throws MojoExecutionException {

        final String artifactKeys[] = JBIAbstractConfigurableMojo
                .getArtifactPatternWithPackaging(artifact);

        for (final String artifactKey : artifactKeys) {

            if (jbiMappingExpressions.containsKey(artifactKey)) {
                return jbiMappingExpressions.getProperty(artifactKey);
            }
        }

        return null;
    }

    /**
     * Find expressions defining the shared libraries of the JBI artifact in the
     * mapping configuration file.
     * 
     * @return The list of shared library Maven artifact as String list,
     *         <code>null</code> if not match found.
     */
    private List<String> getSharedLibrariesMappingExpression(final Artifact artifact)
            throws MojoExecutionException {

        final String artifactKeys[] = JBIAbstractConfigurableMojo
                .getArtifactPatternWithoutPackaging(artifact);
        final List<String> sharedLibraries = new ArrayList<String>();

        for (final String artifactKey : artifactKeys) {
            List<String> removedKeys = new ArrayList<String>();
            for (final Object keyObj : this.jbiSharedLibrariesMappingExpressions.keySet()) {
                final String key = (String) keyObj;

                // We must remove the field "ordering number" before to compare
                // with an artifact pattern
                if (key.substring(0, key.lastIndexOf(':')).equals(artifactKey)) {
                    // We have found a shared library mapping line
                    sharedLibraries.add(this.jbiSharedLibrariesMappingExpressions.getProperty(key));
                    removedKeys.add(key);
                }
            }
            for (String key : removedKeys) {
                this.jbiSharedLibrariesMappingExpressions.remove(key);
            }
            removedKeys.clear();
        }

        return sharedLibraries.size() == 0 ? null : sharedLibraries;
    }

    private static String[] getArtifactPatternWithPackaging(final Artifact artifact) {
        // Array containing all keys corresponfing to a artifact expression
        return new String[] {
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":"
                        + artifact.getVersion() + ":" + artifact.getType(),
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":"
                        + artifact.getVersion() + ":" + "*",
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":" + "*" + ":"
                        + artifact.getType(),
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":" + "*" + ":" + "*",
                artifact.getGroupId() + ":" + "*" + ":" + artifact.getVersion() + ":"
                        + artifact.getType(),
                artifact.getGroupId() + ":" + "*" + ":" + artifact.getVersion() + ":" + "*",
                artifact.getGroupId() + ":" + "*" + ":" + "*" + ":" + artifact.getType(),
                artifact.getGroupId() + ":" + "*" + ":" + "*" + ":" + "*",
                "*" + ":" + artifact.getArtifactId() + ":" + artifact.getVersion() + ":"
                        + artifact.getType(),
                "*" + ":" + artifact.getArtifactId() + ":" + artifact.getVersion() + ":" + "*",
                "*" + ":" + artifact.getArtifactId() + ":" + "*" + ":" + artifact.getType(),
                "*" + ":" + artifact.getArtifactId() + ":" + "*" + ":" + "*",
                "*" + ":" + "*" + ":" + artifact.getVersion() + ":" + artifact.getType(),
                "*" + ":" + "*" + ":" + artifact.getVersion() + ":" + "*",
                "*" + ":" + "*" + ":" + "*" + ":" + artifact.getType(),
                "*" + ":" + "*" + ":" + "*" + ":" + "*" };
    }

    private static String[] getArtifactPatternWithoutPackaging(final Artifact artifact) {
        // Array containing all keys corresponfing to a artifact expression
        return new String[] {
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":"
                        + artifact.getVersion(),
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":"
                        + artifact.getVersion(),
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":" + "*",
                artifact.getGroupId() + ":" + artifact.getArtifactId() + ":" + "*",
                artifact.getGroupId() + ":" + "*" + ":" + artifact.getVersion(),
                artifact.getGroupId() + ":" + "*" + ":" + artifact.getVersion(),
                artifact.getGroupId() + ":" + "*" + ":" + "*",
                artifact.getGroupId() + ":" + "*" + ":" + "*",
                "*" + ":" + artifact.getArtifactId() + ":" + artifact.getVersion(),
                "*" + ":" + artifact.getArtifactId() + ":" + artifact.getVersion(),
                "*" + ":" + artifact.getArtifactId() + ":" + "*",
                "*" + ":" + artifact.getArtifactId() + ":" + "*",
                "*" + ":" + "*" + ":" + artifact.getVersion(),
                "*" + ":" + "*" + ":" + artifact.getVersion(), "*" + ":" + "*" + ":" + "*",
                "*" + ":" + "*" + ":" + "*" };
    }

    /**
     * Read all configuration file needed by the goal "jbi-configure".
     * 
     * @throws MojoExecutionException
     */
    protected void readConfigurationFiles() throws MojoExecutionException {
        this.readJbiIdentifiersMappingFile();
        this.readJbiSharedLibrariesMappingFile();
        this.readJbiVersionsMappingFile();
    }

    /**
     * Read the JBI identifier mapping file.
     * 
     * @throws MojoExecutionException
     */
    private void readJbiIdentifiersMappingFile() throws MojoExecutionException {
        // We look for the mapping file through the classpath
        try {
            this.debug("JBI identifiers mapping configuration file: "
                    + this.jbiIdentifiersMappingFile);
            InputStream isJbiIdentifiersMappingFile = Thread.currentThread()
                    .getContextClassLoader().getResourceAsStream(this.jbiIdentifiersMappingFile);
            if (isJbiIdentifiersMappingFile == null) {
                this.debug("JBI identifiers mapping configuration file: "
                        + this.jbiIdentifiersMappingFileURL);
                // We look for trough the URL
                if (this.jbiIdentifiersMappingFileURL != null) {
                    isJbiIdentifiersMappingFile = this.jbiIdentifiersMappingFileURL.openStream();
                }
            }

            if (isJbiIdentifiersMappingFile == null) {
                this.info("JBI identifiers mapping files not found: "
                        + this.jbiIdentifiersMappingFile + ", URL: "
                        + this.jbiIdentifiersMappingFileURL);
            } else {
                this.jbiIdentifiersMappingExpressions.load(isJbiIdentifiersMappingFile);
            }

        } catch (final IOException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    /**
     * Read the JBI shared libraries mapping file.
     * 
     * @throws MojoExecutionException
     */
    private void readJbiSharedLibrariesMappingFile() throws MojoExecutionException {
        // We look for the mapping file through the classpath
        try {
            this.debug("JBI shared libraries mapping configuration file: "
                    + this.jbiSharedLibrariesMappingFile);
            InputStream isJbiSharedLibrariesMappingFile = Thread.currentThread()
                    .getContextClassLoader()
                    .getResourceAsStream(this.jbiSharedLibrariesMappingFile);
            if (isJbiSharedLibrariesMappingFile == null) {
                // We look for trough the URL
                this.debug("JBI shared libraries mapping configuration file URL: "
                        + this.jbiSharedLibrariesMappingFileURL);
                if (this.jbiSharedLibrariesMappingFileURL != null) {
                    isJbiSharedLibrariesMappingFile = this.jbiSharedLibrariesMappingFileURL
                            .openStream();
                }
            }

            if (isJbiSharedLibrariesMappingFile == null) {
                this.info("JBI shared libraries mapping files not found: "
                        + this.jbiIdentifiersMappingFile + ", URL: "
                        + this.jbiSharedLibrariesMappingFileURL);
            } else {
                this.jbiSharedLibrariesMappingExpressions.load(isJbiSharedLibrariesMappingFile);
            }

        } catch (final IOException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }

    /**
     * Read the JBI versions mapping file.
     * 
     * @throws MojoExecutionException
     */
    private void readJbiVersionsMappingFile() throws MojoExecutionException {
        // We look for the mapping file through the classpath
        try {
            this.debug("JBI versions mapping configuration file: " + this.jbiVersionsMappingFile);
            InputStream isJbiVersionsMappingFile = Thread.currentThread().getContextClassLoader()
                    .getResourceAsStream(this.jbiVersionsMappingFile);
            if (isJbiVersionsMappingFile == null) {
                this.debug("JBI versions mapping configuration file: "
                        + this.jbiVersionsMappingFileURL);
                // We look for trough the URL
                if (this.jbiVersionsMappingFileURL != null) {
                    isJbiVersionsMappingFile = this.jbiVersionsMappingFileURL.openStream();
                }
            }

            if (isJbiVersionsMappingFile == null) {
                this.info("JBI versions mapping files not found: " + this.jbiVersionsMappingFile
                        + ", URL: " + this.jbiVersionsMappingFileURL);
            } else {
                this.jbiVersionsMappingExpressions.load(isJbiVersionsMappingFile);
            }

        } catch (final IOException e) {
            throw new MojoExecutionException(e.getMessage(), e);
        }
    }
}
